<script language="php">
/* ==================================================
Real-Time Game Server Browser 1.0
Copyright 2002 Peter "Spectre1337" Calvache, http://www.planet1337.com
Running on qStat

Use a "body, td" CSS definition to format the fonts

To add map thumbnails, copy the image files into the MAP_THUMBS_DIR directory. Their filename must be equal to the map name which is returned by qStat. Whitespaces in the qStat map name will be replaced by underscores. Image formats are checked in the following order: "jpg", "gif", "png"

All variables that are either important for the configuration of this script or appear multiple times throughout the code are defined as constants below.
================================================== */

// qStat server list file
define( SERVER_FILE, "servers.dat" );
// qstat executable
define( QSTAT_FILE, "/usr/bin/quakestat" );
// map thumbnails directory
define( MAP_THUMBS_DIR, "serverbrowser/mapthumbs" );
// default map thumbnail
define( DEFAULT_MAP_THUMB, "na.jpg" );
// number of items in a rule column
define( RULES_COLUMN_COUNT, 15 );
// alt colors
define( ALT_COLOR_1, "#DFCECF" );
define( ALT_COLOR_2, "#EDDFE0" );
// padding for most tables
define( DEFAULT_PADDING, "2" );
// width of score / ping columns
define( PLAYER_LIST_WIDTH, "50" );
// server info window background
define( SERVER_BACKGROUND, "#F2F2F2" );
// color of the two dividers in the server info window
define( DIVIDER_COLOR, "#FFFFFF" );

class ServerListData
{
	var $serverName;
	var $currentMap;
	var $numberOfPlayers;
	var $maxPlayers;
	var $currentPing;
	var $gameType;
	var $serverAddress;	
}

class ServerInfoData extends ServerListData
{
	var $ruleName;
	var $ruleValue;
	var $players;	
}

function LoadAllData( $serverAddress )
{
	global $serverListData, $serverInfoData;
	
	$serverListData = NULL;
	$serverInfoData = NULL;
	
	exec( QSTAT_FILE." -f ".SERVER_FILE." -u -sort l -raw ,,,", $serverListResult );

	for( $i = 0 ; $i < count( $serverListResult ) ; $i += 2 )
	{
		$partialResult = explode( ",,,", $serverListResult[ $i ] );
		$currentIndex = $i / 2;
		
		$serverListData[ $currentIndex ] = new ServerListData;

		$serverListData[ $currentIndex ]->gameType = $partialResult[ 0 ];
		$serverListData[ $currentIndex ]->serverAddress = $partialResult[ 1 ];
		$serverListData[ $currentIndex ]->serverName = $partialResult[ 2 ];
		$serverListData[ $currentIndex ]->currentMap = $partialResult[ 3 ];
		$serverListData[ $currentIndex ]->maxPlayers = $partialResult[ 4 ];		
		$serverListData[ $currentIndex ]->numberOfPlayers = $partialResult[ 5 ];
		$serverListData[ $currentIndex ]->currentPing = $partialResult[ 6 ];
		
		// optionally load server info data
		if( $serverListData[ $currentIndex ]->serverAddress == $serverAddress )
		{
			$serverInfoData = new ServerInfoData;

			$serverInfoData->gameType = $partialResult[ 0 ];
			$serverInfoData->serverAddress = $partialResult[ 1 ];
			$serverInfoData->serverName = $partialResult[ 2 ];
			$serverInfoData->currentMap = $partialResult[ 3 ];
			$serverInfoData->maxPlayers = $partialResult[ 4 ];		
			$serverInfoData->numberOfPlayers = $partialResult[ 5 ];
			$serverInfoData->currentPing = $partialResult[ 6 ];
			
			exec( QSTAT_FILE." -R -P -".strtolower( $serverListData[ $currentIndex ]->gameType )." ".$serverListData[ $currentIndex ]->serverAddress." -raw ,,,", $serverInfoResult );

			// parse rules
			$partialResult = explode( ",,,", $serverInfoResult[ 1 ] );			
			
			for( $j = 0 ; $j < count( $partialResult ) ; $j++ )
			{
				$ruleResult = explode( "=", $partialResult[ $j ] );
				$serverInfoData->ruleName []= $ruleResult[ 0 ];
				$serverInfoData->ruleValue []= $ruleResult[ 1 ];
			}
			
			// parse players
			for( $j = 2 ; $j < count( $serverInfoResult ) - 1 ; $j++ )
				$serverInfoData->players []= explode( ",,,", $serverInfoResult[ $j ] ); 
		}			
	}
}

function OutputServerInfo()
{
	global $serverInfoData;

	print "<p><table bgcolor=\"".SERVER_BACKGROUND."\" cellpadding=\"10\" cellspacing=\"0\" width=\"100%\" border=\"0\"><tr>\n";

	// server info
	print "<td><div align=\"center\"><font size=\"5\"><b>".$serverInfoData->serverName."</b></font></div><p><hr size=\"1\" color=\"".DIVIDER_COLOR."\"><p><table cellpadding=\"0\"cellspacing=\"0\" width=\"100%\" border=\"0\"><tr><td valign=\"top\" width=\"50%\"><img src=\"serverbrowser/serverinfo.gif\"><p>\n";
	
	print "<b>Address:</b> ".$serverInfoData->serverAddress;

	// map screenshot
	print "<br><b>Current Map:</b><p>\n";
	
	$fileName = str_replace( " ", "_", $serverInfoData->currentMap );
	$fileName = MAP_THUMBS_DIR."/".$fileName;
	
	if( is_file( $fileName.".jpg" ) ) $fileName .= ".jpg";
	else if( is_file( $fileName.".gif" ) ) $fileName .= ".gif";
	else if( is_file( $fileName.".png" ) ) $fileName .= ".png";
	else $fileName = MAP_THUMBS_DIR."/".DEFAULT_MAP_THUMB;
	
	print "<div align=\"center\"><img border=\"1\" src=\"".$fileName."\"><br>".$serverInfoData->currentMap."</div>\n";	

	print "</td>\n";

	// player list
	print "<td valign=\"top\" width=\"50%\">";
	
	if( !$serverInfoData->numberOfPlayers )
	{
		print "This server is currently empty.";
		
	}
	else
	{
		print "<img src=\"serverbrowser/currentlyplaying.gif\"><p>\n";

		print "<div align=\"center\"><b>".$serverInfoData->numberOfPlayers."</b> out of <b>".$serverInfoData->maxPlayers."</b> player slots are occupied.<p>";
		
		print "</div>";
		
		// sort player list
		for( $i = 0 ; $i < count( $serverInfoData->players ) ; $i++ )
		{
			$sortDetails[ 0 ] = $serverInfoData->players[ $i ][ 1 ];
			$sortDetails[ 1 ] = $i;
			$sortArray []= $sortDetails;
		}
		
		for( $i = 0 ; $i < count( $sortArray ) ; $i++ )
		{
			$maxValue = $sortArray[ $i ][ 0 ];
			$maxIndex = $i;	
		
			for( $j = $i ; $j < count( $sortArray ) ; $j++ )
				if( $sortArray[ $j ][ 0 ] > $maxValue )
				{
					$maxValue = $sortArray[ $j ][ 0 ];
					$maxIndex = $j;
				}
				
			$sortedPlayers []= $serverInfoData->players[ $sortArray[ $maxIndex ][ 1 ] ];
			$sortArray[ $maxIndex ] = $sortArray[ $i ];
		}		
	
		$bgColor = false;	
		print "<table cellpadding=\"".DEFAULT_PADDING."\" cellspacing=\"0\" border=\"0\" width=\"100%\"><tr><td width=\"22\"></td><td><b>Player</b></td><td width=\"".PLAYER_LIST_WIDTH."\" align=\"center\"><b>Score</b></td><td width=\"".PLAYER_LIST_WIDTH."\" align=\"center\"><b>Ping</b></td></tr></table>";
		
		for( $i = 0 ; $i < count( $sortedPlayers ) ; $i++ )
		{
			print "<table bgcolor=\"".( $bgColor ? ALT_COLOR_1 : ALT_COLOR_2 )."\" cellpadding=\"".DEFAULT_PADDING."\" cellspacing=\"0\" border=\"0\" width=\"100%\"><tr>";
		
			print "<td width=\"22\">";
			
			if( $i == 0 ) print "<img src=\"serverbrowser\gold.jpg\" border=\"1\">";
			else if( $i == 1 ) print "<img src=\"serverbrowser\silver.jpg\" border=\"1\">";
			else if( $i == 2 ) print "<img src=\"serverbrowser\bronze.jpg\" border=\"1\">";						
			
			print "</td>";
			print "<td>".$sortedPlayers[ $i ][ 0 ]."</td>";
			print "<td width=\"".PLAYER_LIST_WIDTH."\" align=\"center\">".$sortedPlayers[ $i ][ 1 ]."</td>";
			print "<td width=\"".PLAYER_LIST_WIDTH."\" align=\"center\">".$sortedPlayers[ $i ][ 2 ]."</td>";				
			
			print "</tr></table>";
			
			$bgColor = !$bgColor;
		}
	}
	
	print "</td></tr></table></td></tr><tr><td>";	
	
	// rules
	print "<p><hr size=\"1\" color=\"".DIVIDER_COLOR."\"><p><img src=\"serverbrowser/variables.gif\">\n";
	print "<p><table cellpadding=\"0\" cellspacing=\"5\" width=\"100%\" border=\"0\">\n";
	print "<tr>\n";
	
	$bgColor = false;
	
	for( $i = 0, $counter = RULES_COLUMN_COUNT - 1 ; $i < count( $serverInfoData->ruleName ) ; $i++ )
	{
		if( $counter == RULES_COLUMN_COUNT - 1 ) print "<td width=\"".ceil( count( $serverInfoData->ruleName ) / ( RULES_COLUMN_COUNT ) )."%\" valign=\"top\">";
		
		print "<table bgcolor=\"".( $bgColor ? ALT_COLOR_1 : ALT_COLOR_2 )."\" cellpadding=\"".DEFAULT_PADDING."\" cellspacing=\"0\" width=\"100%\" border=\"0\"><tr><td><b><font size=\"1\">".$serverInfoData->ruleName[ $i ]."</font></b></td><td align=\"right\"><font size=\"1\">".$serverInfoData->ruleValue[ $i ]."</font></td></tr></table>\n";
		
		if( $counter == 0 )
		{
			print "</td>";
			$counter = RULES_COLUMN_COUNT - 1;			
			$bgColor = false;
		}
		else
		{
			$counter--;
			$bgColor = !$bgColor;
		}
	}
	
	if( $counter != RULES_COLUMN_COUNT - 1 ) print "</td>";
	
	print "</tr></table>\n";
	
	print "</td></tr>\n";
	print "</table><p>\n";
}

function OutputServerList( $serverAddress )
{
	global $serverListData;

	$bgColor = false;
	print "<img src=\"serverbrowser/servers.gif\"><p><table cellpadding=\"".DEFAULT_PADDING."\" cellspacing=\"0\" border=\"0\" width=\"100%\">\n";
	print "<tr><td><b>Server Name</b></td><td align=\"center\"><b>Current Map</b></td><td align=\"center\"><b>Players</b></td><td align=\"center\"><b>Ping</b></td></tr>\n";

	for( $i = 0 ; $i < count( $serverListData ) ; $i++ )
	{	
		print "<tr bgcolor=\"";
		
		$isHighlighted = false;
		
		// set the row background colors here
		if( $serverListData[ $i ]->serverAddress == $serverAddress )
		{
			print "#C3817F";
			$isHighlighted = true;
		}
		else if( $bgColor ) print ALT_COLOR_1;
		else print ALT_COLOR_2;
		
		print "\">";
		
		print "<td>".( $isHighlighted ? "<img src=\"serverbrowser/arrow.gif\"><b>" : "" )."<a href=\"?viewServer=".$serverListData[ $i ]->serverAddress."\">".$serverListData[ $i ]->serverName."</a>".( $isHighlighted ? "</b>" : "" )."</td>";
		print "<td align=\"center\">".( $isHighlighted ? "<b>" : "" ).$serverListData[ $i ]->currentMap.( $isHighlighted ? "</b>" : "" )."</td>";
		print "<td align=\"center\">".( $isHighlighted ? "<b>" : "" ).$serverListData[ $i ]->numberOfPlayers." / ".$serverListData[ $i ]->maxPlayers.( $isHighlighted ? "</b>" : "" )."</td>";
		print "<td align=\"center\">".( $isHighlighted ? "<b>" : "" ).$serverListData[ $i ]->currentPing.( $isHighlighted ? "</b>" : "" )."</td>";

		print "</tr>\n";
		
		$bgColor = !$bgColor;
	}
	
	print "</table>\n";
}

$serverListData = NULL;
$serverInfoData = NULL;

LoadAllData( $viewServer );

if( $viewServer ) OutputServerInfo();
else print "<div align=\"center\">Click on a server name to view advanced information.</div><p>\n";

OutputServerList( $viewServer );

</script>
